/**
 * Copyright (C) 2022 Xilinx, Inc
 * Copyright (C) 2022 Advanced Micro Devices, Inc. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may
 * not use this file except in compliance with the License. A copy of the
 * License is located at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

// ------ I N C L U D E   F I L E S -------------------------------------------
// Local - Include Files
#include "ReportErrorCnt.h"
#include "MAUtilities.h"

// 3rd Party Library - Include Files
#include <string>
#include <boost/format.hpp>
#include <boost/algorithm/string.hpp>

#define BYTES_TO_MEGABYTES 0x100000ll

void ReportErrorCnt::getPropertyTree(const std::shared_ptr<MA35Device> dev, boost::property_tree::ptree& _pt) const {
  boost::property_tree::ptree pt;

  XBUtilities::get_error_info(dev, pt);
  // There can only be 1 root node
  _pt.add_child("error_cnt", pt);
}

void ReportErrorCnt::writeReport(const boost::property_tree::ptree& _pt, std::ostream& _output) const {
  boost::property_tree::ptree empty_ptree;

  _output << "MA35 Error Counter Info:\n";
  if (_pt.get<std::string>("error_cnt.status") != "ok") {
    _output << "ERROR " << _pt.get<std::string>("error_cnt.status") << std::endl;
    return;
  }

  try {
    _output << boost::format("    %-25s%-20s%-20s\n") % "     Tag" % "Uncorrectable" % "Correctable";

    const auto& errors = _pt.get_child("error_cnt.error_cnt", empty_ptree);
    for (const auto& kd : errors) {
      const auto& t1 = kd.second;
      const auto  c1 = t1.get<std::string>("uncorrectable", "?");
      const auto  c2 = t1.get<std::string>("correctable", "?");
      _output << boost::format("    %-25s%-20s%-20s\n") % kd.first % c1 % c2;
    }
  } catch (const boost::property_tree::ptree_error& ex) {
    _output << "ERROR: Unexpected error" << std::endl;
  }
  _output << std::endl;
}
