// Copyright (C) 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#pragma once

#include "Device.h"

class DeviceVoltage : public jsf_helpers::NodeTelemetry {
private:
  static constexpr uint32_t PVT_VREF = 1291; // 1.291 V
  const Device&             m_device;
  unsigned                  m_index;
  unsigned                  m_offset;

public:
  DeviceVoltage(const Device& Device, const std::string& Name, const std::string& Location, const std::string& Unit, unsigned Index, unsigned Offset, bool Demo) :
      NodeTelemetry(Name, Location, Unit, Demo), m_device(Device), m_index(Index), m_offset(Offset) {
      }

  virtual json GetValue(bool /* Demo */) override;
};

class DeviceTemperature : public jsf_helpers::NodeTelemetry {
private:
  static constexpr uint32_t PVT_DOTP25 = 1000;
  const Device&             m_device;
  unsigned                  m_index;
  unsigned                  m_offset;

public:
  DeviceTemperature(const Device& Device, const std::string& Name, const std::string& Location, const std::string& Unit, unsigned Index, unsigned Offset, bool Demo) :
      NodeTelemetry(Name, Location, Unit, Demo), m_device(Device), m_index(Index), m_offset(Offset) {
      }

  virtual json GetValue(bool) override;
};

class SensorInfoNode : public jsf_helpers::NodeTelemetry {
private:
  const Device& m_device;
  unsigned      m_index;
  unsigned      m_offset;

public:
  SensorInfoNode(const Device& Device, const std::string& Name, const std::string& Location, const std::string& Unit, unsigned Index, unsigned Offset, bool Demo) :
      NodeTelemetry(Name, Location, Unit, Demo), m_device(Device), m_index(Index), m_offset(Offset) {
      }

  virtual json GetValue(bool) override;
};

class BoardVcNode : public jsf_helpers::NodeTelemetry {
private:
  const Device& m_device;
  unsigned      m_index;
  unsigned      m_offset;

public:
  BoardVcNode(const Device& Device, const std::string& Name, const std::string& Location, const std::string& Unit, unsigned Index, unsigned Offset, bool Demo) :
      NodeTelemetry(Name, Location, Unit, Demo), m_device(Device), m_index(Index), m_offset(Offset) {
      }

  virtual json GetValue(bool) override;
};

class BoardPower : public jsf_helpers::NodeTelemetry {
private:
  const Device& m_device;
  unsigned      m_index;
  unsigned      m_offset;

public:
  BoardPower(const Device& Device, const std::string& Name, const std::string& Location, const std::string& Unit, unsigned Index, unsigned Offset, bool Demo) :
      NodeTelemetry(Name, Location, Unit, Demo), m_device(Device), m_index(Index), m_offset(Offset) {
      }

  virtual json GetValue(bool) override;
};

class Uptime : public jsf_int::NodeLeaf {
private:
  const Device& m_device;
  unsigned      m_offset;

public:
  Uptime(const Device& Device, const std::string& Name, unsigned Offset) : NodeLeaf(Name), m_device(Device), m_offset(Offset) {
  }
  virtual json Invoke(const json& Parameters, bool Iteration) override;
};

class VersionInfo : public jsf_int::NodeLeaf {
private:
  const Device& m_device;
  unsigned      m_index;
  unsigned      m_offset;

public:
  VersionInfo(const Device& Device, const std::string& Name, unsigned Index, unsigned Offset) : NodeLeaf(Name), m_device(Device), m_index(Index), m_offset(Offset) {
  }

  virtual json Invoke(const json& Parameters, bool Iteration) override;
};

class ProductInfoNode : public jsf_int::NodeLeaf {
private:
  const Device& m_device;
  unsigned      m_index;
  unsigned      m_offset;

public:
  ProductInfoNode(const Device& Device, const std::string& Name, unsigned Index, unsigned Offset) : NodeLeaf(Name), m_device(Device), m_index(Index), m_offset(Offset) {
  }

  virtual json Invoke(const json& Parameters, bool Iteration) override;
};

class PcieInfo : public jsf_helpers::NodeRawFile {
private:
  Device& m_device;

public:
  PcieInfo(Device& Device, const std::string& Name, const std::string& SysfsFile) :
      NodeRawFile(Name, "/sys/bus/pci/devices/" + Device.GetBdf() + "/" + SysfsFile), m_device(Device) {
      }

  virtual json Invoke(const json& Parameters, bool Iteration) override;
};

class SkuNumber : public ProductInfoNode {
public:
  SkuNumber(const Device& Device, const std::string& Name, unsigned Index, unsigned Offset) : ProductInfoNode(Device, Name, Index, Offset) {
  }

  virtual json Invoke(const json& Parameters, bool Iteration) override;
};

class DdrInfoNode : public jsf_int::NodeLeaf {
private:
  const Device& m_device;
  unsigned      m_index;
  unsigned      m_offset;

public:
  DdrInfoNode(const Device& Device, const std::string& Name, unsigned Index, unsigned Offset) : NodeLeaf(Name), m_device(Device), m_index(Index), m_offset(Offset) {
  }

  virtual json Invoke(const json& Parameters, bool Iteration) override;
};

class ErrorCounters : public jsf_int::NodeLeaf {
private:
  const Device& m_device;
  unsigned      m_index;
  unsigned      m_offset;

public:
  ErrorCounters(const Device& Device, const std::string& Name, unsigned Index, unsigned Offset) : NodeLeaf(Name), m_device(Device), m_index(Index), m_offset(Offset) {
  }

  virtual json Invoke(const json& Parameters, bool Iteration) override;
};
